//////////////////////////////////////////////////////////////////////////////////////
//																					//
//								FishinoServer.h										//
//						Library for ESP8266 WiFi module								//
//					Created by Massimo Del Fedele, 2015								//
//																					//
//  Copyright (c) 2015, 2016 and 2017 Massimo Del Fedele.  All rights reserved.		//
//																					//
//	Redistribution and use in source and binary forms, with or without				//
//	modification, are permitted provided that the following conditions are met:		//
//																					//
//	- Redistributions of source code must retain the above copyright notice,		//
//	  this list of conditions and the following disclaimer.							//
//	- Redistributions in binary form must reproduce the above copyright notice,		//
//	  this list of conditions and the following disclaimer in the documentation		//
//	  and/or other materials provided with the distribution.						//
//																					//	
//	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"		//
//	AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE		//
//	IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE		//
//	ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE		//
//	LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR				//
//	CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF			//
//	SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS		//
//	INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN			//
//	CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)			//
//	ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE		//
//	POSSIBILITY OF SUCH DAMAGE.														//
//																					//
//	VERSION 1.0.0 - INITIAL VERSION													//
//	VERSION 2.0.0 - 06/01/2016 - REWROTE SPI INTERFACE AND ERROR HANDLING			//
//	VERSION 4.0.0 - 01/01/2017 - REWROTE SPI INTERFACE AND ERROR HANDLING			//
//	VERSION 5.1.0 - 04/05/2017 - USE NEW DEBUG LIBRARY								//
//	VERSION 5.2.0 - 20/05/2017 - USE NEW DEBUG LIBRARY								//
//	Version 6.0.0 -- June 2017 - USE NEW DEBUG LIBRARY								//
//	Version 7.0.0 -- June 2017 - REWROTE SPI INTERFACE								//
//	Version 7.0.1 -- June 2017 - FIXED BUG ON MEGA									//
//	Version 7.1.0 - 20/10/2017 - AVOID AUTOMATIC SOCKET CLOSE FOR DELETED CLIENTS	//
//	Version 7.3.0 - 17/12/2017 - ADD WIFI MODULE DEEP SLEEP COMMAND					//
//																					//
//////////////////////////////////////////////////////////////////////////////////////
#ifndef FISHINOSERVER_H
#define FISHINOSERVER_H

#include "Server.h"

class FishinoServer : public Server
{
	template<class S, class C> friend class RefCounted;

	private:
		uint16_t _port;
		bool _accepting;
		
		// reference counting for copied objects
		RefCounted<uint8_t, FishinoServer> *ref;
		
		// remove socket on last reference drop
		static void killSocket(uint8_t sock) { Fishino.stopServer(sock); }

	public:
		FishinoServer(uint16_t port);
		
		~FishinoServer();
		
		virtual bool hasClients();
		FishinoClient available();
		
		virtual void begin(void);
		virtual void close(void) { stop() ; }
		virtual void stop(void);
		
		virtual void setNoDelay(bool n);
		virtual bool getNoDelay(void);
		
		virtual size_t write(uint8_t);
		virtual size_t write(const uint8_t *buf, size_t size);
		using Print::write;

		// set server buffered mode
		bool setBufferedMode(bool b);
		
		// get server buffered mode
		bool getBufferedMode(void);
		
		// set server clients timeout
		bool setClientsForceCloseTime(uint32_t tim);
		
		// get server clients timeout
		uint32_t getClientsForceCloseTime(void);
		
		// set max number of server clients
		bool setMaxClients(uint8_t n);
		
		// get max number of server clients
		uint8_t getMaxClients(void);
};

#endif
